# =====================================
# generator=datazen
# version=3.1.2
# hash=48a9f16af16af88514e495c1a03fb18b
# =====================================
"""
rcmpy - Package's default entry-point.
"""

# built-in
import sys

# internal
from rcmpy.entry import main

if __name__ == "__main__":
    sys.exit(main(sys.argv))
