# =====================================
# generator=datazen
# version=3.1.2
# hash=c3c3f1e4bd14e6c9f1861b5a32713a39
# =====================================

"""
Useful defaults and other package metadata.
"""

DESCRIPTION = "A configuration-file management system."
PKG_NAME = "rcmpy"
VERSION = "1.4.1"
